@@ -1,13 +1,17 @@
 /*
- * DEBUG: section 16    Cache Manager API
+ * Copyright (C) 1996-2018 The Squid Software Foundation and contributors
  *
+ * Squid software is distributed under GPLv2+ license and includes
+ * contributions from numerous individuals and organizations.
+ * Please see the COPYING and CONTRIBUTORS files for details.
  */
 
+/* DEBUG: section 16    Cache Manager API */
+
 #ifndef SQUID_MGR_COUNTERS_ACTION_H
 #define SQUID_MGR_COUNTERS_ACTION_H
 
 #include "mgr/Action.h"
-#include <sys/time.h>
 
 namespace Mgr
 {
@@ -103,3 +107,4 @@ private:
 } // namespace Mgr
 
 #endif /* SQUID_MGR_COUNTERS_ACTION_H */
+
